@@ -2,12 +2,15 @@
 #
 # SPDX-License-Identifier: GPL-3.0-or-later
 
+# pylint: disable=too-many-positional-arguments
+
 """Command dispatcher for TabbedBrowser."""
 
 import os.path
 import shlex
 import functools
-from typing import cast, Callable, Dict, Union, Optional
+from typing import cast, Union, Optional
+from collections.abc import Callable
 
 from qutebrowser.qt.widgets import QApplication, QTabBar
 from qutebrowser.qt.core import Qt, QUrl, QEvent, QUrlQuery
@@ -636,7 +639,7 @@ class CommandDispatcher:
         widget = self._current_widget()
         url = self._current_url()
 
-        handlers: Dict[str, Callable[..., QUrl]] = {
+        handlers: dict[str, Callable[..., QUrl]] = {
             'prev': functools.partial(navigate.prevnext, prev=True),
             'next': functools.partial(navigate.prevnext, prev=False),
             'up': navigate.path_up,
@@ -698,28 +701,6 @@ class CommandDispatcher:
                 "Numeric argument is too large for internal int "
                 "representation.")
 
-    def _yank_url(self, what):
-        """Helper method for yank() to get the URL to copy."""
-        assert what in ['url', 'pretty-url'], what
-
-        if what == 'pretty-url':
-            flags = urlutils.FormatOption.DECODE_RESERVED
-        else:
-            flags = urlutils.FormatOption.ENCODED
-        flags |= urlutils.FormatOption.REMOVE_PASSWORD
-
-        url = QUrl(self._current_url())
-        url_query = QUrlQuery()
-        url_query_str = url.query()
-        if '&' not in url_query_str and ';' in url_query_str:
-            url_query.setQueryDelimiters('=', ';')
-        url_query.setQuery(url_query_str)
-        for key in dict(url_query.queryItems()):
-            if key in config.val.url.yank_ignored_parameters:
-                url_query.removeQueryItem(key)
-        url.setQuery(url_query)
-        return url.toString(flags)
-
     @cmdutils.register(instance='command-dispatcher', scope='window')
     @cmdutils.argument('what', choices=['selection', 'url', 'pretty-url',
                                         'title', 'domain', 'inline'])
@@ -754,7 +735,9 @@ class CommandDispatcher:
                                    self._current_url().host(),
                                    ':' + str(port) if port > -1 else '')
         elif what in ['url', 'pretty-url']:
-            s = self._yank_url(what)
+            url = self._current_url()
+            pretty = what == 'pretty-url'
+            s = urlutils.get_url_yank_text(url, pretty=pretty)
             what = 'URL'  # For printing
         elif what == 'selection':
             def _selection_callback(s):
@@ -1156,8 +1139,7 @@ class CommandDispatcher:
         else:
             cmd = os.path.expanduser(cmd)
             proc = guiprocess.GUIProcess(what='command', verbose=verbose,
-                                         output_messages=output_messages,
-                                         parent=self._tabbed_browser)
+                                         output_messages=output_messages)
             if detach:
                 ok = proc.start_detached(cmd, args)
                 if not ok:
