@@ -2,19 +2,19 @@
 #
 # SPDX-License-Identifier: GPL-3.0-or-later
 
-"""Wrapper over a QWebEngineView."""
+"""Wrapper over a WebEngineView."""
 
 import math
+import struct
 import functools
 import dataclasses
 import re
 import html as html_utils
 from typing import cast, Union, Optional
 
-from qutebrowser.qt.core import (pyqtSignal, pyqtSlot, Qt, QPoint, QPointF, QTimer, QUrl,
-                          QObject)
+from qutebrowser.qt.core import (pyqtSignal, pyqtSlot, Qt, QPoint, QPointF, QUrl,
+                          QObject, QByteArray)
 from qutebrowser.qt.network import QAuthenticator
-from qutebrowser.qt.webenginewidgets import QWebEngineView
 from qutebrowser.qt.webenginecore import QWebEnginePage, QWebEngineScript, QWebEngineHistory
 
 from qutebrowser.config import config
@@ -292,6 +292,8 @@ class WebEngineCaret(browsertab.AbstractCaret):
         flags = set()
         if utils.is_windows:
             flags.add('windows')
+        if 'caret' in objects.debug_flags:
+            flags.add('debug')
         return list(flags)
 
     @pyqtSlot(usertypes.KeyMode)
@@ -612,8 +614,16 @@ class WebEngineHistoryPrivate(browsertab.AbstractHistoryPrivate):
         self._tab = tab
         self._history = cast(QWebEngineHistory, None)
 
+    def _serialize_data(self, stream_version, count, current_index):
+        return struct.pack(">IIi", stream_version, count, current_index)
+
     def serialize(self):
-        return qtutils.serialize(self._history)
+        data = qtutils.serialize(self._history)
+        # WORKAROUND for https://bugreports.qt.io/browse/QTBUG-117489
+        if data == self._serialize_data(stream_version=4, count=1, current_index=0):
+            fixed = self._serialize_data(stream_version=4, count=0, current_index=-1)
+            return QByteArray(fixed)
+        return data
 
     def deserialize(self, data):
         qtutils.deserialize(data, self._history)
@@ -806,7 +816,7 @@ class WebEngineAudio(browsertab.AbstractAudio):
         # Implements the intended two-second delay specified at
         # https://doc.qt.io/archives/qt-5.14/qwebenginepage.html#recentlyAudibleChanged
         delay_ms = 2000
-        self._silence_timer = QTimer(self)
+        self._silence_timer = usertypes.Timer(self)
         self._silence_timer.setSingleShot(True)
         self._silence_timer.setInterval(delay_ms)
 
@@ -816,6 +826,8 @@ class WebEngineAudio(browsertab.AbstractAudio):
         page.recentlyAudibleChanged.connect(self._delayed_recently_audible_changed)
         self._tab.url_changed.connect(self._on_url_changed)
         config.instance.changed.connect(self._on_config_changed)
+        self._silence_timer.timeout.connect(functools.partial(
+            self.recently_audible_changed.emit, False))
 
     # WORKAROUND for recentlyAudibleChanged being emitted without delay from the moment
     # that audio is dropped.
@@ -831,8 +843,6 @@ class WebEngineAudio(browsertab.AbstractAudio):
             # Ignore all subsequent calls while the tab is muted with an active timer
             if timer.isActive():
                 return
-            timer.timeout.connect(
-                functools.partial(self.recently_audible_changed.emit, recently_audible))
             timer.start()
 
     def set_muted(self, muted: bool, override: bool = False) -> None:
@@ -876,6 +886,8 @@ class _WebEnginePermissions(QObject):
         QWebEnginePage.Feature.MouseLock: 'content.mouse_lock',
         QWebEnginePage.Feature.DesktopVideoCapture: 'content.desktop_capture',
         QWebEnginePage.Feature.DesktopAudioVideoCapture: 'content.desktop_capture',
+        # 8 == ClipboardReadWrite, new in 6.8
+        QWebEnginePage.Feature(8): 'content.javascript.clipboard',
     }
 
     _messages = {
@@ -887,6 +899,7 @@ class _WebEnginePermissions(QObject):
         QWebEnginePage.Feature.MouseLock: 'hide your mouse pointer',
         QWebEnginePage.Feature.DesktopVideoCapture: 'capture your desktop',
         QWebEnginePage.Feature.DesktopAudioVideoCapture: 'capture your desktop and audio',
+        QWebEnginePage.Feature(8): 'read and write your clipboard',
     }
 
     def __init__(self, tab, parent=None):
@@ -1258,7 +1271,7 @@ class WebEngineTab(browsertab.AbstractTab):
 
     abort_questions = pyqtSignal()
 
-    _widget: QWebEngineView
+    _widget: webview.WebEngineView
     search: WebEngineSearch
     audio: WebEngineAudio
     printing: WebEnginePrinting
@@ -1467,9 +1480,9 @@ class WebEngineTab(browsertab.AbstractTab):
             log.network.debug("Asking for credentials")
             answer = shared.authentication_required(
                 url, authenticator, abort_on=[self.abort_questions])
-        if not netrc_success and answer is None:
-            log.network.debug("Aborting auth")
-            sip.assign(authenticator, QAuthenticator())
+            if answer is None:
+                log.network.debug("Aborting auth")
+                sip.assign(authenticator, QAuthenticator())
 
     @pyqtSlot()
     def _on_load_started(self):
@@ -1503,7 +1516,7 @@ class WebEngineTab(browsertab.AbstractTab):
                 browsertab.TerminationStatus.crashed,
             QWebEnginePage.RenderProcessTerminationStatus.KilledTerminationStatus:
                 browsertab.TerminationStatus.killed,
-            -1:
+            QWebEnginePage.RenderProcessTerminationStatus(-1):
                 browsertab.TerminationStatus.unknown,
         }
         self.renderer_process_terminated.emit(status_map[status], exitcode)
