@@ -1,21 +1,6 @@
-# vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
-
-# Copyright 2014-2021 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# SPDX-FileCopyrightText: Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
-# This file is part of qutebrowser.
-#
-# qutebrowser is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# qutebrowser is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with qutebrowser.  If not, see <https://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-or-later
 
 """Configuration files residing on disk."""
 
@@ -29,11 +14,11 @@ import traceback
 import configparser
 import contextlib
 import re
-from typing import (TYPE_CHECKING, Any, Dict, Iterable, Iterator, List, Mapping,
-                    MutableMapping, Optional, Tuple, cast)
+from typing import (TYPE_CHECKING, Any, Optional, cast)
+from collections.abc import Iterable, Iterator, Mapping, MutableMapping
 
 import yaml
-from PyQt5.QtCore import pyqtSignal, pyqtSlot, QObject, QSettings, qVersion
+from qutebrowser.qt.core import pyqtSignal, pyqtSlot, QObject, QSettings, qVersion
 
 import qutebrowser
 from qutebrowser.config import (configexc, config, configdata, configutils,
@@ -49,7 +34,7 @@ if TYPE_CHECKING:
 state = cast('StateConfig', None)
 
 
-_SettingsType = Dict[str, Dict[str, Any]]
+_SettingsType = dict[str, dict[str, Any]]
 
 
 class VersionChange(enum.Enum):
@@ -70,7 +55,7 @@ class VersionChange(enum.Enum):
         This is intended to use filters like "major" (show major only), "minor" (show
         major/minor) or "patch" (show all changes).
         """
-        allowed_values: Dict[str, List[VersionChange]] = {
+        allowed_values: dict[str, list[VersionChange]] = {
             'major': [VersionChange.major],
             'minor': [VersionChange.major, VersionChange.minor],
             'patch': [VersionChange.major, VersionChange.minor, VersionChange.patch],
@@ -91,6 +76,7 @@ class StateConfig(configparser.ConfigParser):
         self.qt_version_changed = False
         self.qtwe_version_changed = False
         self.qutebrowser_version_changed = VersionChange.unknown
+        self.chromium_version_changed = VersionChange.unknown
         self._set_changed_attributes()
 
         for sect in ['general', 'geometry', 'inspector']:
@@ -103,25 +89,50 @@ class StateConfig(configparser.ConfigParser):
             ('general', 'fooled'),
             ('general', 'backend-warning-shown'),
             ('general', 'old-qt-warning-shown'),
+            ('general', 'serviceworker_workaround'),
             ('geometry', 'inspector'),
         ]
         for sect, key in deleted_keys:
             self[sect].pop(key, None)
 
-        self['general']['qt_version'] = qVersion()
+        qt_version = qVersion()
+        assert qt_version is not None
+        self['general']['qt_version'] = qt_version
         self['general']['qtwe_version'] = self._qtwe_version_str()
+        self['general']['chromium_version'] = self._chromium_version_str()
         self['general']['version'] = qutebrowser.__version__
 
-    def _qtwe_version_str(self) -> str:
-        """Get the QtWebEngine version string.
+    def _has_webengine(self) -> bool:
+        """Check if QtWebEngine is available.
 
         Note that it's too early to use objects.backend here...
         """
         try:
-            import PyQt5.QtWebEngineWidgets  # pylint: disable=unused-import
+            # pylint: disable=unused-import,redefined-outer-name
+            import qutebrowser.qt.webenginewidgets
         except ImportError:
+            return False
+        return True
+
+    def _qtwe_versions(self) -> Optional[version.WebEngineVersions]:
+        """Get the QtWebEngine versions."""
+        if not self._has_webengine():
+            return None
+        return version.qtwebengine_versions(avoid_init=True)
+
+    def _qtwe_version_str(self) -> str:
+        """Get the QtWebEngine version string."""
+        versions = self._qtwe_versions()
+        if versions is None:
             return 'no'
-        return str(version.qtwebengine_versions(avoid_init=True).webengine)
+        return str(versions.webengine)
+
+    def _chromium_version_str(self) -> str:
+        """Get the Chromium major version string."""
+        versions = self._qtwe_versions()
+        if versions is None:
+            return 'no'
+        return str(versions.chromium_major)
 
     def _set_changed_attributes(self) -> None:
         """Set qt_version_changed/qutebrowser_version_changed attributes.
@@ -139,10 +150,14 @@ class StateConfig(configparser.ConfigParser):
         old_qtwe_version = self['general'].get('qtwe_version', None)
         self.qtwe_version_changed = old_qtwe_version != self._qtwe_version_str()
 
+        self._set_qutebrowser_changed_attribute()
+        self._set_chromium_changed_attribute()
+
+    def _set_qutebrowser_changed_attribute(self) -> None:
+        """Detect a qutebrowser version change."""
         old_qutebrowser_version = self['general'].get('version', None)
         if old_qutebrowser_version is None:
-            # https://github.com/python/typeshed/issues/2093
-            return  # type: ignore[unreachable]
+            return
 
         try:
             old_version = utils.VersionNumber.parse(old_qutebrowser_version)
@@ -163,6 +178,46 @@ class StateConfig(configparser.ConfigParser):
         else:
             self.qutebrowser_version_changed = VersionChange.major
 
+    def _set_chromium_changed_attribute(self) -> None:
+        if not self._has_webengine():
+            return
+
+        old_chromium_version_str = self['general'].get('chromium_version', None)
+        if old_chromium_version_str in ['no', None]:
+            old_qtwe_version = self['general'].get('qtwe_version', None)
+            if old_qtwe_version in ['no', None]:
+                return
+
+            try:
+                old_chromium_version = version.WebEngineVersions.from_webengine(
+                    old_qtwe_version, source='config').chromium_major
+            except ValueError:
+                log.init.warning(
+                    f"Unable to parse old QtWebEngine version {old_qtwe_version}")
+                return
+        else:
+            try:
+                old_chromium_version = int(old_chromium_version_str)
+            except ValueError:
+                log.init.warning(
+                    f"Unable to parse old Chromium version {old_chromium_version_str}")
+                return
+
+        new_versions = version.qtwebengine_versions(avoid_init=True)
+        new_chromium_version = new_versions.chromium_major
+
+        if old_chromium_version is None or new_chromium_version is None:
+            return
+
+        if old_chromium_version <= 87 and new_chromium_version >= 90:  # Qt 5 -> Qt 6
+            self.chromium_version_changed = VersionChange.major
+        elif old_chromium_version > new_chromium_version:
+            self.chromium_version_changed = VersionChange.downgrade
+        elif old_chromium_version == new_chromium_version:
+            self.chromium_version_changed = VersionChange.equal
+        else:
+            self.chromium_version_changed = VersionChange.minor
+
     def init_save_manager(self,
                           save_manager: 'savemanager.SaveManager') -> None:
         """Make sure the config gets saved properly.
@@ -195,7 +250,7 @@ class YamlConfig(QObject):
                                       'autoconfig.yml')
         self._dirty = False
 
-        self._values: Dict[str, configutils.Values] = {}
+        self._values: dict[str, configutils.Values] = {}
         for name, opt in configdata.DATA.items():
             self._values[name] = configutils.Values(opt)
 
@@ -238,7 +293,7 @@ class YamlConfig(QObject):
             f.write(textwrap.dedent("""
                 # If a config.py file exists, this file is ignored unless it's explicitly loaded
                 # via config.load_autoconfig(). For more information, see:
-                # https://github.com/qutebrowser/qutebrowser/blob/master/doc/help/configuring.asciidoc#loading-autoconfigyml
+                # https://github.com/qutebrowser/qutebrowser/blob/main/doc/help/configuring.asciidoc#loading-autoconfigyml
                 # DO NOT edit this file by hand, qutebrowser will overwrite it.
                 # Instead, create a config.py - see :help for details.
 
@@ -384,8 +439,8 @@ class YamlMigrations(QObject):
 
     changed = pyqtSignal()
 
-    def __init__(self, settings: _SettingsType,
-                 parent: QObject = None) -> None:
+    # Note: settings is Any because it's not validated yet.
+    def __init__(self, settings: Any, parent: QObject = None) -> None:
         super().__init__(parent)
         self._settings = settings
 
@@ -422,6 +477,12 @@ class YamlMigrations(QObject):
             false_value='load-insecurely',
             ask_value='ask',
         )
+        self._migrate_renamed_bool(
+            old_name='content.javascript.can_access_clipboard',
+            new_name='content.javascript.clipboard',
+            true_value='access',
+            false_value='none',
+        )
 
         for setting in ['colors.webpage.force_dark_color_scheme',
                         'colors.webpage.prefers_color_scheme_dark']:
@@ -641,13 +702,13 @@ class ConfigAPI:
     ):
         self._config = conf
         self._keyconfig = keyconfig
-        self.errors: List[configexc.ConfigErrorDesc] = []
+        self.errors: list[configexc.ConfigErrorDesc] = []
         self.configdir = pathlib.Path(standarddir.config())
         self.datadir = pathlib.Path(standarddir.data())
         self._warn_autoconfig = warn_autoconfig
 
     @contextlib.contextmanager
-    def _handle_error(self, action: str, name: str) -> Iterator[None]:
+    def _handle_error(self, action: str) -> Iterator[None]:
         """Catch config-related exceptions and save them in self.errors."""
         try:
             yield
@@ -656,13 +717,13 @@ class ConfigAPI:
                 new_err = err.with_text(e.basename)
                 self.errors.append(new_err)
         except configexc.Error as e:
-            text = "While {} '{}'".format(action, name)
+            text = f"While {action}"
             self.errors.append(configexc.ConfigErrorDesc(text, e))
         except urlmatch.ParseError as e:
-            text = "While {} '{}' and parsing pattern".format(action, name)
+            text = f"While {action} and parsing pattern"
             self.errors.append(configexc.ConfigErrorDesc(text, e))
         except keyutils.KeyParseError as e:
-            text = "While {} '{}' and parsing key".format(action, name)
+            text = f"While {action} and parsing key"
             self.errors.append(configexc.ConfigErrorDesc(text, e))
 
     def finalize(self) -> None:
@@ -674,30 +735,32 @@ class ConfigAPI:
                  " (to load settings configured via the GUI) or "
                  "`config.load_autoconfig(False)` (to not do so)"))
             self.errors.append(desc)
-        self._config.update_mutables()
+
+        with self._handle_error("updating mutated values"):
+            self._config.update_mutables()
 
     def load_autoconfig(self, load_config: bool = True) -> None:
         """Load the autoconfig.yml file which is used for :set/:bind/etc."""
         self._warn_autoconfig = False
         if load_config:
-            with self._handle_error('reading', 'autoconfig.yml'):
+            with self._handle_error("reading 'autoconfig.yml'"):
                 read_autoconfig()
 
     def get(self, name: str, pattern: str = None) -> Any:
         """Get a setting value from the config, optionally with a pattern."""
-        with self._handle_error('getting', name):
+        with self._handle_error(f"getting '{name}'"):
             urlpattern = urlmatch.UrlPattern(pattern) if pattern else None
             return self._config.get_mutable_obj(name, pattern=urlpattern)
 
     def set(self, name: str, value: Any, pattern: str = None) -> None:
         """Set a setting value in the config, optionally with a pattern."""
-        with self._handle_error('setting', name):
+        with self._handle_error(f"setting '{name}'"):
             urlpattern = urlmatch.UrlPattern(pattern) if pattern else None
             self._config.set_obj(name, value, pattern=urlpattern)
 
     def bind(self, key: str, command: Optional[str], mode: str = 'normal') -> None:
         """Bind a key to a command, with an optional key mode."""
-        with self._handle_error('binding', key):
+        with self._handle_error(f"binding '{key}'"):
             seq = keyutils.KeySequence.parse(key)
             if command is None:
                 raise configexc.Error("Can't bind {key} to None (maybe you "
@@ -707,7 +770,7 @@ class ConfigAPI:
 
     def unbind(self, key: str, mode: str = 'normal') -> None:
         """Unbind a key from a command, with an optional key mode."""
-        with self._handle_error('unbinding', key):
+        with self._handle_error(f"unbinding '{key}'"):
             seq = keyutils.KeySequence.parse(key)
             self._keyconfig.unbind(seq, mode=mode)
 
@@ -740,8 +803,8 @@ class ConfigPyWriter:
 
     def __init__(
             self,
-            options: List[
-                Tuple[
+            options: list[
+                tuple[
                     Optional[urlmatch.UrlPattern],
                     configdata.Option,
                     Any
@@ -982,5 +1045,5 @@ def init() -> None:
     # https://github.com/qutebrowser/qutebrowser/issues/515
 
     path = os.path.join(standarddir.config(auto=True), 'qsettings')
-    for fmt in [QSettings.NativeFormat, QSettings.IniFormat]:
-        QSettings.setPath(fmt, QSettings.UserScope, path)
+    for fmt in [QSettings.Format.NativeFormat, QSettings.Format.IniFormat]:
+        QSettings.setPath(fmt, QSettings.Scope.UserScope, path)
