@@ -1,21 +1,6 @@
-# vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
-
-# Copyright 2014-2021 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# SPDX-FileCopyrightText: Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
-# This file is part of qutebrowser.
-#
-# qutebrowser is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# qutebrowser is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with qutebrowser.  If not, see <https://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-or-later
 
 """Utilities to show various version information."""
 
@@ -26,7 +11,6 @@ import os.path
 import platform
 import subprocess
 import importlib
-import collections
 import pathlib
 import configparser
 import enum
@@ -34,29 +18,32 @@ import datetime
 import getpass
 import functools
 import dataclasses
-from typing import (Mapping, Optional, Sequence, Tuple, ClassVar, Dict, cast,
+import importlib.metadata
+from typing import (Optional, ClassVar, Any,
                     TYPE_CHECKING)
+from collections.abc import Mapping, Sequence
 
-
-from PyQt5.QtCore import PYQT_VERSION_STR, QLibraryInfo, qVersion
-from PyQt5.QtNetwork import QSslSocket
-from PyQt5.QtGui import (QOpenGLContext, QOpenGLVersionProfile,
-                         QOffscreenSurface)
-from PyQt5.QtWidgets import QApplication
+from qutebrowser.qt import machinery
+from qutebrowser.qt.core import PYQT_VERSION_STR
+from qutebrowser.qt.network import QSslSocket
+from qutebrowser.qt.gui import QOpenGLContext, QOffscreenSurface
+from qutebrowser.qt.opengl import QOpenGLVersionProfile
+from qutebrowser.qt.widgets import QApplication
 
 try:
-    from PyQt5.QtWebKit import qWebKitVersion
+    from qutebrowser.qt.webkit import qWebKitVersion
 except ImportError:  # pragma: no cover
     qWebKitVersion = None  # type: ignore[assignment]  # noqa: N816
 try:
-    from PyQt5.QtWebEngine import PYQT_WEBENGINE_VERSION_STR
+    from qutebrowser.qt.webenginecore import PYQT_WEBENGINE_VERSION_STR
 except ImportError:  # pragma: no cover
-    # Added in PyQt 5.13
+    # QtWebKit
     PYQT_WEBENGINE_VERSION_STR = None  # type: ignore[assignment]
 
 
 import qutebrowser
-from qutebrowser.utils import log, utils, standarddir, usertypes, message, resources
+from qutebrowser.utils import (log, utils, standarddir, usertypes, message, resources,
+                               qtutils)
 from qutebrowser.misc import objects, earlyinit, sql, httpclient, pastebin, elf
 from qutebrowser.browser import pdfjs
 from qutebrowser.config import config
@@ -92,7 +79,7 @@ class DistributionInfo:
     pretty: str
 
 
-pastebin_url = None
+pastebin_url: Optional[str] = None
 
 
 class Distribution(enum.Enum):
@@ -119,7 +106,7 @@ class Distribution(enum.Enum):
     solus = enum.auto()
 
 
-def _parse_os_release() -> Optional[Dict[str, str]]:
+def _parse_os_release() -> Optional[dict[str, str]]:
     """Parse an /etc/os-release file."""
     filename = os.environ.get('QUTE_FAKE_OS_RELEASE', '/etc/os-release')
     info = {}
@@ -264,7 +251,7 @@ def _git_str_subprocess(gitpath: str) -> Optional[str]:
         return None
 
 
-def _release_info() -> Sequence[Tuple[str, str]]:
+def _release_info() -> Sequence[tuple[str, str]]:
     """Try to gather distribution release information.
 
     Return:
@@ -393,22 +380,39 @@ class ModuleInfo:
         return text
 
 
-MODULE_INFO: Mapping[str, ModuleInfo] = collections.OrderedDict([
-    # FIXME: Mypy doesn't understand this. See https://github.com/python/mypy/issues/9706
-    (name, ModuleInfo(name, *args))  # type: ignore[arg-type, misc]
-    for (name, *args) in
-    [
-        ('sip', ['SIP_VERSION_STR']),
+def _create_module_info() -> dict[str, ModuleInfo]:
+    packages = [
         ('colorama', ['VERSION', '__version__']),
         ('jinja2', ['__version__']),
         ('pygments', ['__version__']),
         ('yaml', ['__version__']),
         ('adblock', ['__version__'], "0.3.2"),
-        ('PyQt5.QtWebEngineWidgets', []),
-        ('PyQt5.QtWebEngine', ['PYQT_WEBENGINE_VERSION_STR']),
-        ('PyQt5.QtWebKitWidgets', []),
+        ('objc', ['__version__']),
     ]
-])
+
+    if machinery.IS_QT5:
+        packages += [
+            ('PyQt5.QtWebEngineWidgets', []),
+            ('PyQt5.QtWebEngine', ['PYQT_WEBENGINE_VERSION_STR']),
+            ('PyQt5.QtWebKitWidgets', []),
+            ('PyQt5.sip', ['SIP_VERSION_STR']),
+        ]
+    elif machinery.IS_QT6:
+        packages += [
+            ('PyQt6.QtWebEngineCore', ['PYQT_WEBENGINE_VERSION_STR']),
+            ('PyQt6.sip', ['SIP_VERSION_STR']),
+        ]
+    else:
+        raise utils.Unreachable()
+
+    # Mypy doesn't understand this. See https://github.com/python/mypy/issues/9706
+    return {
+        name: ModuleInfo(name, *args)  # type: ignore[arg-type, misc]
+        for (name, *args) in packages
+    }
+
+
+MODULE_INFO: Mapping[str, ModuleInfo] = _create_module_info()
 
 
 def _module_versions() -> Sequence[str]:
@@ -477,13 +481,13 @@ def _pdfjs_version() -> str:
         A string with the version number.
     """
     try:
-        pdfjs_file, file_path = pdfjs.get_pdfjs_res_and_path('build/pdf.js')
+        pdfjs_file, file_path = pdfjs.get_pdfjs_res_and_path(pdfjs.get_pdfjs_js_path())
     except pdfjs.PDFJSNotFound:
         return 'no'
     else:
         pdfjs_file = pdfjs_file.decode('utf-8')
         version_re = re.compile(
-            r"^ *(PDFJS\.version|(var|const) pdfjsVersion) = '(?P<version>[^']+)';$",
+            r"""^ *(PDFJS\.version|(var|const) pdfjsVersion) = ['"](?P<version>[^'"]+)['"];$""",
             re.MULTILINE)
 
         match = version_re.search(pdfjs_file)
@@ -506,24 +510,20 @@ def _get_pyqt_webengine_qt_version() -> Optional[str]:
     PyQtWebEngine 5.15.4 renamed it to PyQtWebEngine-Qt5...:
     https://www.riverbankcomputing.com/pipermail/pyqt/2021-March/043699.html
 
-    Here, we try to use importlib.metadata or its backport (optional dependency) to
-    figure out that version number. If PyQtWebEngine is installed via pip, this will
-    give us an accurate answer.
+    Here, we try to use importlib.metadata to figure out that version number.
+    If PyQtWebEngine is installed via pip, this will give us an accurate answer.
     """
-    try:
-        import importlib.metadata as importlib_metadata  # type: ignore[import]
-    except ImportError:
-        try:
-            import importlib_metadata  # type: ignore[no-redef]
-        except ImportError:
-            log.misc.debug("Neither importlib.metadata nor backport available")
-            return None
+    names = (
+        ['PyQt6-WebEngine-Qt6']
+        if machinery.IS_QT6 else
+        ['PyQtWebEngine-Qt5', 'PyQtWebEngine-Qt']
+    )
 
-    for suffix in ['Qt5', 'Qt']:
+    for name in names:
         try:
-            return importlib_metadata.version(f'PyQtWebEngine-{suffix}')
-        except importlib_metadata.PackageNotFoundError:
-            log.misc.debug(f"PyQtWebEngine-{suffix} not found")
+            return importlib.metadata.version(name)
+        except importlib.metadata.PackageNotFoundError:
+            log.misc.debug(f"{name} not found")
 
     return None
 
@@ -536,49 +536,107 @@ class WebEngineVersions:
     webengine: utils.VersionNumber
     chromium: Optional[str]
     source: str
+    chromium_security: Optional[str] = None
     chromium_major: Optional[int] = dataclasses.field(init=False)
 
-    _CHROMIUM_VERSIONS: ClassVar[Dict[utils.VersionNumber, str]] = {
+    # Dates based on https://chromium.googlesource.com/chromium/src/+refs
+    _BASES: ClassVar[dict[int, str]] = {
+        83: '83.0.4103.122',  # 2020-06-27, Qt 5.15.2
+        87: '87.0.4280.144',  # 2021-01-08, Qt 5.15
+        90: '90.0.4430.228',  # 2021-06-22, Qt 6.2
+        94: '94.0.4606.126',  # 2021-11-17, Qt 6.3
+        102: '102.0.5005.177',  # 2022-09-01, Qt 6.4
+        # (.220 claimed by code, .181 claimed by CHROMIUM_VERSION)
+        108: '108.0.5359.220',  # 2023-01-27, Qt 6.5
+        112: '112.0.5615.213',  # 2023-05-24, Qt 6.6
+        118: '118.0.5993.220',  # 2024-01-25, Qt 6.7
+        122: '122.0.6261.171',  # 2024-04-15, Qt 6.8
+    }
+
+    # Dates based on https://chromereleases.googleblog.com/
+    _CHROMIUM_VERSIONS: ClassVar[dict[utils.VersionNumber, tuple[str, Optional[str]]]] = {
+        # ====== UNSUPPORTED =====
+
         # Qt 5.12: Chromium 69
         # (LTS)    69.0.3497.128 (~2018-09-11)
-        #          5.12.0: Security fixes up to 70.0.3538.102 (~2018-10-24)
-        #          5.12.1: Security fixes up to 71.0.3578.94  (2018-12-12)
-        #          5.12.2: Security fixes up to 72.0.3626.121 (2019-03-01)
-        #          5.12.3: Security fixes up to 73.0.3683.75  (2019-03-12)
-        #          5.12.4: Security fixes up to 74.0.3729.157 (2019-05-14)
-        #          5.12.5: Security fixes up to 76.0.3809.87  (2019-07-30)
-        #          5.12.6: Security fixes up to 77.0.3865.120 (~2019-09-10)
-        #          5.12.7: Security fixes up to 79.0.3945.130 (2020-01-16)
-        #          5.12.8: Security fixes up to 80.0.3987.149 (2020-03-18)
-        #          5.12.9: Security fixes up to 83.0.4103.97  (2020-06-03)
         #          5.12.10: Security fixes up to 86.0.4240.75 (2020-10-06)
-        utils.VersionNumber(5, 12): '69.0.3497.128',
 
         # Qt 5.13: Chromium 73
         #          73.0.3683.105 (~2019-02-28)
-        #          5.13.0: Security fixes up to 74.0.3729.157 (2019-05-14)
-        #          5.13.1: Security fixes up to 76.0.3809.87  (2019-07-30)
         #          5.13.2: Security fixes up to 77.0.3865.120 (2019-10-10)
-        utils.VersionNumber(5, 13): '73.0.3683.105',
 
         # Qt 5.14: Chromium 77
         #          77.0.3865.129 (~2019-10-10)
-        #          5.14.0: Security fixes up to 77.0.3865.129 (~2019-09-10)
-        #          5.14.1: Security fixes up to 79.0.3945.117 (2020-01-07)
         #          5.14.2: Security fixes up to 80.0.3987.132 (2020-03-03)
-        utils.VersionNumber(5, 14): '77.0.3865.129',
 
         # Qt 5.15: Chromium 80
         #          80.0.3987.163 (2020-04-02)
         #          5.15.0: Security fixes up to 81.0.4044.138 (2020-05-05)
         #          5.15.1: Security fixes up to 85.0.4183.83  (2020-08-25)
-        #          5.15.2: Updated to 83.0.4103.122           (~2020-06-24)
-        #                  Security fixes up to 86.0.4240.183 (2020-11-02)
-        #          5.15.3: Updated to 87.0.4280.144           (~2020-12-02)
-        #                  Security fixes up to 88.0.4324.150 (2021-02-04)
-        utils.VersionNumber(5, 15): '80.0.3987.163',
-        utils.VersionNumber(5, 15, 2): '83.0.4103.122',
-        utils.VersionNumber(5, 15, 3): '87.0.4280.144',
+
+        # ====== SUPPORTED =====
+        #                               base         security
+        ## Qt 5.15
+        utils.VersionNumber(5, 15, 2): (_BASES[83], '86.0.4240.183'),  # 2020-11-02
+        utils.VersionNumber(5, 15): (_BASES[87], None),  # >= 5.15.3
+        utils.VersionNumber(5, 15, 3): (_BASES[87], '88.0.4324.150'),  # 2021-02-04
+        # 5.15.4 to 5.15.6: unknown security fixes
+        utils.VersionNumber(5, 15, 7): (_BASES[87], '94.0.4606.61'),  # 2021-09-24
+        utils.VersionNumber(5, 15, 8): (_BASES[87], '96.0.4664.110'),  # 2021-12-13
+        utils.VersionNumber(5, 15, 9): (_BASES[87], '98.0.4758.102'),  # 2022-02-14
+        utils.VersionNumber(5, 15, 10): (_BASES[87], '98.0.4758.102'),  # (?) 2022-02-14
+        utils.VersionNumber(5, 15, 11): (_BASES[87], '98.0.4758.102'),  # (?) 2022-02-14
+        utils.VersionNumber(5, 15, 12): (_BASES[87], '98.0.4758.102'),  # (?) 2022-02-14
+        utils.VersionNumber(5, 15, 13): (_BASES[87], '108.0.5359.124'),  # 2022-12-13
+        utils.VersionNumber(5, 15, 14): (_BASES[87], '113.0.5672.64'),  # 2023-05-02
+        # 5.15.15: unknown security fixes
+        utils.VersionNumber(5, 15, 16): (_BASES[87], '119.0.6045.123'),  # 2023-11-07
+        utils.VersionNumber(5, 15, 17): (_BASES[87], '123.0.6312.58'),  # 2024-03-19
+        utils.VersionNumber(5, 15, 18): (_BASES[87], '130.0.6723.59'),  # 2024-10-14
+
+
+        ## Qt 6.2
+        utils.VersionNumber(6, 2): (_BASES[90], '93.0.4577.63'),  # 2021-08-31
+        utils.VersionNumber(6, 2, 1): (_BASES[90], '94.0.4606.61'),  # 2021-09-24
+        utils.VersionNumber(6, 2, 2): (_BASES[90], '96.0.4664.45'),  # 2021-11-15
+        utils.VersionNumber(6, 2, 3): (_BASES[90], '96.0.4664.45'),  # 2021-11-15
+        utils.VersionNumber(6, 2, 4): (_BASES[90], '98.0.4758.102'),  # 2022-02-14
+        # 6.2.5 / 6.2.6: unknown security fixes
+        utils.VersionNumber(6, 2, 7): (_BASES[90], '107.0.5304.110'),  # 2022-11-08
+        utils.VersionNumber(6, 2, 8): (_BASES[90], '111.0.5563.110'),  # 2023-03-21
+
+        ## Qt 6.3
+        utils.VersionNumber(6, 3): (_BASES[94], '99.0.4844.84'),  # 2022-03-25
+        utils.VersionNumber(6, 3, 1): (_BASES[94], '101.0.4951.64'),  # 2022-05-10
+        utils.VersionNumber(6, 3, 2): (_BASES[94], '104.0.5112.81'),  # 2022-08-01
+
+        ## Qt 6.4
+        utils.VersionNumber(6, 4): (_BASES[102], '104.0.5112.102'),  # 2022-08-16
+        utils.VersionNumber(6, 4, 1): (_BASES[102], '107.0.5304.88'),  # 2022-10-27
+        utils.VersionNumber(6, 4, 2): (_BASES[102], '108.0.5359.94'),  # 2022-12-02
+        utils.VersionNumber(6, 4, 3): (_BASES[102], '110.0.5481.78'),  # 2023-02-07
+
+        ## Qt 6.5
+        utils.VersionNumber(6, 5): (_BASES[108], '110.0.5481.104'),  # 2023-02-16
+        utils.VersionNumber(6, 5, 1): (_BASES[108], '112.0.5615.138'),  # 2023-04-18
+        utils.VersionNumber(6, 5, 2): (_BASES[108], '114.0.5735.133'),  # 2023-06-13
+        utils.VersionNumber(6, 5, 3): (_BASES[108], '117.0.5938.63'),  # 2023-09-12
+
+        ## Qt 6.6
+        utils.VersionNumber(6, 6): (_BASES[112], '117.0.5938.63'),  # 2023-09-12
+        utils.VersionNumber(6, 6, 1): (_BASES[112], '119.0.6045.123'),  # 2023-11-07
+        utils.VersionNumber(6, 6, 2): (_BASES[112], '121.0.6167.160'),  # 2024-02-06
+        utils.VersionNumber(6, 6, 3): (_BASES[112], '122.0.6261.128'),  # 2024-03-12
+
+        ## Qt 6.7
+        utils.VersionNumber(6, 7): (_BASES[118], '122.0.6261.128'),  # 2024-03-12
+        utils.VersionNumber(6, 7, 1): (_BASES[118], '124.0.6367.202'),  # 2024-05-09
+        utils.VersionNumber(6, 7, 2): (_BASES[118], '125.0.6422.142'),  # 2024-05-30
+        utils.VersionNumber(6, 7, 3): (_BASES[118], '129.0.6668.58'),  # 2024-09-17
+
+        ## Qt 6.8
+        utils.VersionNumber(6, 8): (_BASES[122], '129.0.6668.70'),  # 2024-09-24
+        utils.VersionNumber(6, 8, 1): (_BASES[122], '131.0.6778.70'),  # 2024-11-12
     }
 
     def __post_init__(self) -> None:
@@ -589,25 +647,37 @@ class WebEngineVersions:
             self.chromium_major = int(self.chromium.split('.')[0])
 
     def __str__(self) -> str:
-        s = f'QtWebEngine {self.webengine}'
+        lines = [f'QtWebEngine {self.webengine}']
         if self.chromium is not None:
-            s += f', based on Chromium {self.chromium}'
-        if self.source != 'UA':
-            s += f' (from {self.source})'
-        return s
+            lines.append(f'  based on Chromium {self.chromium}')
+        if self.chromium_security is not None:
+            lines.append(f'  with security patches up to {self.chromium_security} (plus any distribution patches)')
+        lines.append(f'  (source: {self.source})')
+        return "\n".join(lines)
 
     @classmethod
     def from_ua(cls, ua: 'websettings.UserAgent') -> 'WebEngineVersions':
         """Get the versions parsed from a user agent.
 
-        This is the most reliable and "default" way to get this information (at least
-        until QtWebEngine adds an API for it). However, it needs a fully initialized
-        QtWebEngine, and we sometimes need this information before that is available.
+        This is the most reliable and "default" way to get this information for
+        older Qt versions that don't provide an API for it. However, it needs a
+        fully initialized QtWebEngine, and we sometimes need this information
+        before that is available.
         """
         assert ua.qt_version is not None, ua
+        webengine = utils.VersionNumber.parse(ua.qt_version)
+        chromium_inferred, chromium_security = cls._infer_chromium_version(webengine)
+        if ua.upstream_browser_version != chromium_inferred:  # pragma: no cover
+            # should never happen, but let's play it safe
+            log.misc.debug(
+                f"Chromium version mismatch: {ua.upstream_browser_version} (UA) != "
+                f"{chromium_inferred} (inferred)")
+            chromium_security = None
+
         return cls(
-            webengine=utils.VersionNumber.parse(ua.qt_version),
+            webengine=webengine,
             chromium=ua.upstream_browser_version,
+            chromium_security=chromium_security,
             source='UA',
         )
 
@@ -622,9 +692,19 @@ class WebEngineVersions:
         sometimes mix and match Qt/QtWebEngine versions, so this is a more reliable
         (though hackish) way to get a more accurate result.
         """
+        webengine = utils.VersionNumber.parse(versions.webengine)
+        chromium_inferred, chromium_security = cls._infer_chromium_version(webengine)
+        if versions.chromium != chromium_inferred:  # pragma: no cover
+            # should never happen, but let's play it safe
+            log.misc.debug(
+                f"Chromium version mismatch: {versions.chromium} (ELF) != "
+                f"{chromium_inferred} (inferred)")
+            chromium_security = None
+
         return cls(
-            webengine=utils.VersionNumber.parse(versions.webengine),
+            webengine=webengine,
             chromium=versions.chromium,
+            chromium_security=chromium_security,
             source='ELF',
         )
 
@@ -632,25 +712,56 @@ class WebEngineVersions:
     def _infer_chromium_version(
             cls,
             pyqt_webengine_version: utils.VersionNumber,
-    ) -> Optional[str]:
-        """Infer the Chromium version based on the PyQtWebEngine version."""
-        chromium_version = cls._CHROMIUM_VERSIONS.get(pyqt_webengine_version)
+    ) -> tuple[Optional[str], Optional[str]]:
+        """Infer the Chromium version based on the PyQtWebEngine version.
+
+        Returns:
+            A tuple of the Chromium version and the security patch version.
+        """
+        chromium_version, security_version = cls._CHROMIUM_VERSIONS.get(
+            pyqt_webengine_version, (None, None))
         if chromium_version is not None:
-            return chromium_version
+            return chromium_version, security_version
 
         # 5.15 patch versions change their QtWebEngine version, but no changes are
-        # expected after 5.15.3.
-        v5_15_3 = utils.VersionNumber(5, 15, 3)
-        if v5_15_3 <= pyqt_webengine_version < utils.VersionNumber(6):
-            minor_version = v5_15_3
-        else:
-            # e.g. 5.14.2 -> 5.14
-            minor_version = pyqt_webengine_version.strip_patch()
+        # expected after 5.15.3 and 5.15.[01] are unsupported.
+        assert pyqt_webengine_version != utils.VersionNumber(5, 15, 2)
 
-        return cls._CHROMIUM_VERSIONS.get(minor_version)
+        # e.g. 5.15.4 -> 5.15
+        # we ignore the security version as that one will have changed from .0
+        # and is thus unknown.
+        minor_version = pyqt_webengine_version.strip_patch()
+        chromium_ver, _security_ver = cls._CHROMIUM_VERSIONS.get(
+            minor_version, (None, None))
+
+        return chromium_ver, None
 
     @classmethod
-    def from_importlib(cls, pyqt_webengine_qt_version: str) -> 'WebEngineVersions':
+    def from_api(
+        cls,
+        qtwe_version: str,
+        chromium_version: Optional[str],
+        chromium_security: Optional[str] = None,
+    ) -> 'WebEngineVersions':
+        """Get the versions based on the exact versions.
+
+        This is called if we have proper APIs to get the versions easily
+        (Qt 6.2 with PyQt 6.3.1+).
+        """
+        parsed = utils.VersionNumber.parse(qtwe_version)
+        return cls(
+            webengine=parsed,
+            chromium=chromium_version,
+            chromium_security=chromium_security,
+            source='api',
+        )
+
+    @classmethod
+    def from_webengine(
+        cls,
+        pyqt_webengine_qt_version: str,
+        source: str,
+    ) -> 'WebEngineVersions':
         """Get the versions based on the PyQtWebEngine version.
 
         This is called if we don't want to fully initialize QtWebEngine (so
@@ -658,14 +769,16 @@ class WebEngineVersions:
         a PyQtWebEngine-Qt{,5} package from PyPI, so we could query its exact version.
         """
         parsed = utils.VersionNumber.parse(pyqt_webengine_qt_version)
+        chromium, chromium_security = cls._infer_chromium_version(parsed)
         return cls(
             webengine=parsed,
-            chromium=cls._infer_chromium_version(parsed),
-            source='importlib',
+            chromium=chromium,
+            chromium_security=chromium_security,
+            source=source,
         )
 
     @classmethod
-    def from_pyqt(cls, pyqt_webengine_version: str) -> 'WebEngineVersions':
+    def from_pyqt(cls, pyqt_webengine_version: str, source: str = "PyQt") -> 'WebEngineVersions':
         """Get the versions based on the PyQtWebEngine version.
 
         This is the "last resort" if we don't want to fully initialize QtWebEngine (so
@@ -703,22 +816,12 @@ class WebEngineVersions:
             if frozen:
                 parsed = utils.VersionNumber(5, 15, 2)
 
+        chromium, chromium_security = cls._infer_chromium_version(parsed)
+
         return cls(
             webengine=parsed,
-            chromium=cls._infer_chromium_version(parsed),
-            source='PyQt',
-        )
-
-    @classmethod
-    def from_qt(cls, qt_version: str, *, source: str = 'Qt') -> 'WebEngineVersions':
-        """Get the versions based on the Qt version.
-
-        This is called if we don't have PYQT_WEBENGINE_VERSION, i.e. with PyQt 5.12.
-        """
-        parsed = utils.VersionNumber.parse(qt_version)
-        return cls(
-            webengine=parsed,
-            chromium=cls._infer_chromium_version(parsed),
+            chromium=chromium,
+            chromium_security=chromium_security,
             source=source,
         )
 
@@ -742,6 +845,35 @@ def qtwebengine_versions(*, avoid_init: bool = False) -> WebEngineVersions:
     - https://www.chromium.org/developers/calendar
     - https://chromereleases.googleblog.com/
     """
+    override = os.environ.get('QUTE_QTWEBENGINE_VERSION_OVERRIDE')
+    if override is not None:
+        return WebEngineVersions.from_pyqt(override, source='override')
+
+    if machinery.IS_QT6:
+        try:
+            from qutebrowser.qt.webenginecore import (
+                qWebEngineVersion,
+                qWebEngineChromiumVersion,
+            )
+        except ImportError:
+            pass  # Needs QtWebEngine 6.2+ with PyQtWebEngine 6.3.1+
+        else:
+            try:
+                from qutebrowser.qt.webenginecore import (
+                    qWebEngineChromiumSecurityPatchVersion,
+                )
+                chromium_security = qWebEngineChromiumSecurityPatchVersion()
+            except ImportError:
+                chromium_security = None  # Needs QtWebEngine 6.3+
+
+            qtwe_version = qWebEngineVersion()
+            assert qtwe_version is not None
+            return WebEngineVersions.from_api(
+                qtwe_version=qtwe_version,
+                chromium_version=qWebEngineChromiumVersion(),
+                chromium_security=chromium_security,
+            )
+
     from qutebrowser.browser.webengine import webenginesettings
 
     if webenginesettings.parsed_user_agent is None and not avoid_init:
@@ -750,22 +882,17 @@ def qtwebengine_versions(*, avoid_init: bool = False) -> WebEngineVersions:
     if webenginesettings.parsed_user_agent is not None:
         return WebEngineVersions.from_ua(webenginesettings.parsed_user_agent)
 
-    override = os.environ.get('QUTE_QTWEBENGINE_VERSION_OVERRIDE')
-    if override is not None:
-        return WebEngineVersions.from_qt(override, source='override')
-
     versions = elf.parse_webenginecore()
     if versions is not None:
         return WebEngineVersions.from_elf(versions)
 
     pyqt_webengine_qt_version = _get_pyqt_webengine_qt_version()
     if pyqt_webengine_qt_version is not None:
-        return WebEngineVersions.from_importlib(pyqt_webengine_qt_version)
+        return WebEngineVersions.from_webengine(
+            pyqt_webengine_qt_version, source='importlib')
 
-    if PYQT_WEBENGINE_VERSION_STR is not None:
-        return WebEngineVersions.from_pyqt(PYQT_WEBENGINE_VERSION_STR)
-
-    return WebEngineVersions.from_qt(qVersion())  # type: ignore[unreachable]
+    assert PYQT_WEBENGINE_VERSION_STR is not None
+    return WebEngineVersions.from_pyqt(PYQT_WEBENGINE_VERSION_STR)
 
 
 def _backend() -> str:
@@ -814,6 +941,8 @@ def version_info() -> str:
                         platform.python_version()),
         'PyQt: {}'.format(PYQT_VERSION_STR),
         '',
+        str(machinery.INFO),
+        '',
     ]
 
     lines += _module_versions()
@@ -827,7 +956,10 @@ def version_info() -> str:
 
     if objects.qapp:
         style = objects.qapp.style()
-        lines.append('Style: {}'.format(style.metaObject().className()))
+        assert style is not None
+        metaobj = style.metaObject()
+        assert metaobj is not None
+        lines.append('Style: {}'.format(metaobj.className()))
         lines.append('Platform plugin: {}'.format(objects.qapp.platformName()))
         lines.append('OpenGL: {}'.format(opengl_info()))
 
@@ -848,8 +980,8 @@ def version_info() -> str:
         "Imported from {}".format(importpath),
         "Using Python from {}".format(sys.executable),
         "Qt library executable path: {}, data path: {}".format(
-            QLibraryInfo.location(QLibraryInfo.LibraryExecutablesPath),
-            QLibraryInfo.location(QLibraryInfo.DataPath)
+            qtutils.library_path(qtutils.LibraryPath.library_executables),
+            qtutils.library_path(qtutils.LibraryPath.data),
         )
     ]
 
@@ -890,7 +1022,7 @@ class OpenGLInfo:
     version_str: Optional[str] = None
 
     # The parsed version as a (major, minor) tuple of ints
-    version: Optional[Tuple[int, ...]] = None
+    version: Optional[tuple[int, ...]] = None
 
     # The vendor specific information following the version number
     vendor_specific: Optional[str] = None
@@ -946,7 +1078,7 @@ def opengl_info() -> Optional[OpenGLInfo]:  # pragma: no cover
         vendor, version = override.split(', ', maxsplit=1)
         return OpenGLInfo.parse(vendor=vendor, version=version)
 
-    old_context = cast(Optional[QOpenGLContext], QOpenGLContext.currentContext())
+    old_context: Optional[QOpenGLContext] = QOpenGLContext.currentContext()
     old_surface = None if old_context is None else old_context.surface()
 
     surface = QOffscreenSurface()
@@ -972,7 +1104,12 @@ def opengl_info() -> Optional[OpenGLInfo]:  # pragma: no cover
         vp.setVersion(2, 0)
 
         try:
-            vf = ctx.versionFunctions(vp)
+            if machinery.IS_QT5:
+                vf = ctx.versionFunctions(vp)
+            else:
+                # Qt 6
+                from qutebrowser.qt.opengl import QOpenGLVersionFunctionsFactory
+                vf: Any = QOpenGLVersionFunctionsFactory.get(vp, ctx)
         except ImportError as e:
             log.init.debug("Importing version functions failed: {}".format(e))
             return None
@@ -981,6 +1118,7 @@ def opengl_info() -> Optional[OpenGLInfo]:  # pragma: no cover
             log.init.debug("Getting version functions failed!")
             return None
 
+        # FIXME:mypy PyQt6-stubs issue?
         vendor = vf.glGetString(vf.GL_VENDOR)
         version = vf.glGetString(vf.GL_VERSION)
 
